/*
  license GPL v3 ...........

  description of this file:
  a page for viewing a user selected RSS feed ;

*/

import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import "./dateutils.js" as DateUtils
import "databasemodule_v2.js" as DB

Page {
    id: page_feed

    property string rssModelUrl: ""
    property string rssTitle: ""
    property int rssIndex: 0
    property var rssModel
    property var rssItem: null
    property bool preventIndexChangeHandler: false

    onRssIndexChanged: {
        rssListview.currentIndex = rssIndex
        rssListview.positionViewAtIndex(rssListview.currentIndex, ListView.Center)
    }

    function setFeed(model, index) {
        /* Setting new model and not-null index will cause two change events instead of one.
         * Settings "preventIndexChangeHandler" to true helps to avoid it.
         */
        if (rssModel != model && index !== 0)
            preventIndexChangeHandler = true
        rssModel = model
        rssIndex = index
    }

    function reloadPageContent() {

    }

    tools: ToolbarItems {
        id: toolbar

        ToolbarButton {
            id: readingOptionsButton
            /* QtRoS - Set visibility to true in line below to see options popup.
             * There are no guides on dark and light theme yet, as on font size.
             * So it is just logic for storing settings without any visual effects.
             */
            visible: false
            action: Action {
                text:  i18n.tr("Options")
                iconSource: Qt.resolvedUrl("./icons_tmp/settings.svg")
                onTriggered:
                {
                    PopupUtils.open(readingOptionsPopoverComponent, readingOptionsButton)
                }
            }
        }

        ToolbarButton {
            action: Action {
                text:  rssItem == null ? "" : (rssItem.favourite == "0" ? i18n.tr("Save") : i18n.tr("Unsave"))
                iconSource: {
                    if (rssItem == null || rssItem.favourite == "0")
                        return Qt.resolvedUrl("./icons_tmp/favorite-unselected.svg")
                    else return Qt.resolvedUrl("./icons_tmp/favorite-selected.svg")
                }
                onTriggered: {
                    var fav = (rssItem.favourite == "0" ? "1" : "0")
                    var dbResult = DB.updateArticleFavourite(rssItem.id, fav)
                    if (dbResult.rowsAffected == 1) {
                        rssItem.favourite = fav
                        var dbResult1 = DB.updateArticleStatus(rssItem.id, "0")
                        if (dbResult1.rowsAffected == 1) {
                            rssItem.status = "0"
                        }
                        mainView.refreshSavedTab()
                    }
                }
            }
        }

        ToolbarButton {
            action: Action {
                text:  i18n.tr("Open site")
                iconSource: Qt.resolvedUrl("./icons_tmp/go-to.svg")
                onTriggered:
                {
                    console.log("Open site", rssListview.model.get(rssListview.currentIndex).link)
                    Qt.openUrlExternally(rssListview.model.get(rssListview.currentIndex).link)
                }
            }
        }
    }

    Component {
        id: readingOptionsPopoverComponent

        Popover {
            id: readingOptionsPopover

            Component.onCompleted: {
//                var useDarkTheme = DB.getUseDarkTheme()
//                console.log("USE DARK", useDarkTheme)
                fontSizeSlider.value = optionsKeeper.fontSize()
                buttonRow.updateButtonsState()
            }

            Column {
                id: contentColumn

                anchors {
                    left: parent.left
                    top: parent.top
                    right: parent.right
                }

                ListItem.Empty {
                    Row {
                        id: buttonRow

                        property bool useDark: false
                        spacing: units.gu(2)
                        anchors.centerIn: parent
                        Button {
                            text: "Dark"
                            width: units.gu(14)
                            gradient: buttonRow.useDark ? UbuntuColors.orangeGradient : UbuntuColors.greyGradient
                            onClicked: {
                                optionsKeeper.setUseDarkTheme(true)
                                buttonRow.updateButtonsState()
                            }
                        }

                        Button {
                            text: "Light"
                            width: units.gu(14)
                            gradient: buttonRow.useDark ? UbuntuColors.greyGradient : UbuntuColors.orangeGradient
                            onClicked: {
                                optionsKeeper.setUseDarkTheme(false)
                                buttonRow.updateButtonsState()
                            }
                        }

                        function updateButtonsState() {
                            useDark = optionsKeeper.useDarkTheme()
                        }
                    }
                }

                ListItem.Empty {
                    showDivider: false
                    Label {
                        id: lblLess

                        text: "A"
                        fontSize: "small"
                        color: "black" // TODO TEMP
                        anchors {
                            left: parent.left
                            leftMargin: units.gu(2)
                            verticalCenter: parent.verticalCenter
                        }
                    }

                    Slider {
                        id: fontSizeSlider

                        anchors {
                            right: lblMore.left
                            rightMargin: units.gu(1)
                            left: lblLess.right
                            leftMargin: units.gu(1)
                            verticalCenter: parent.verticalCenter
                        }

                        minimumValue: 0
                        maximumValue: 2

                        onValueChanged: {
                            var res
                            if (value < maximumValue / 3)
                                res = 0
                            else if (value < maximumValue / 3 * 2)
                                res = 1
                            else res = 2

                            optionsKeeper.setFontSize(res)
                        }

                        function formatValue(v) {
                            if (v < maximumValue / 3)
                                return i18n.tr("Small")
                            else if (v < maximumValue / 3 * 2)
                                return i18n.tr("Mid")
                            else return i18n.tr("Large")
                        }
                    } // SLider

                    Label {
                        id: lblMore

                        text: "A"
                        fontSize: "large"
                        color: "black" // TODO TEMP
                        anchors {
                            right: parent.right
                            rightMargin: units.gu(2)
                            verticalCenter: parent.verticalCenter
                        }
                    }
                } // ListItem.Empty
            }
        }
    } // Component

    //////////////////////////////////////////////      a listview to show the RSS content
    ListView {
        id: rssListview

        width: parent.width
        anchors.bottom: parent.bottom
        anchors.bottomMargin: units.gu(2)
        anchors.top: parent.top
        anchors.topMargin: units.gu(2)
        snapMode: ListView.SnapOneItem
        cacheBuffer: 90
        boundsBehavior: Flickable.StopAtBounds
        orientation: ListView.Horizontal
        contentHeight: parent.width * count
        model: rssModel
        delegate: xmlDelegate
        highlightFollowsCurrentItem: true
        highlightRangeMode: ListView.StrictlyEnforceRange

        onCurrentIndexChanged: {
            console.log("ListView onCurrentIndexChanged", currentIndex, preventIndexChangeHandler)
            if (preventIndexChangeHandler) {
                preventIndexChangeHandler = false
                return
            }
            rssItem = rssModel.get(currentIndex)
            rssTitle = rssItem.feed_name

            if (rssItem.status != "1") {
                var dbResult = DB.updateArticleStatus(rssItem.id, "1")
                if (dbResult.rowsAffected == 1) {
                    rssItem.status = "1"
                }
            }
        }
    }

    //////////////////////////////////////////////      delegate for ListView
    Component {
        id: xmlDelegate

        Flickable {
            id: scrollArea

            clip: true

            width: rssListview.width
            height: rssListview.height

            contentWidth: width
            contentHeight: innerAreaColumn.height

            Column {
                id: innerAreaColumn

                spacing: units.gu(2)
                width: parent.width

                Label {
                    id: label_time
                    text: DateUtils.formatRelativeTime(i18n, pubdate)
                    fontSize: "small"
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                }

                Label {
                    fontSize: "large"
                    text: title
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                }

                Label {
                    text: content
                    linkColor: UbuntuColors.orange // Temporary. Mb bad color, but better than dark blue.
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                    clip: true
                }

                Label {
                    id: label_feedname
                    text: rssTitle
                    fontSize: "small"
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                }
            }
        } // Flickable
    } // Component
}
