import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../dateutils.js" as DateUtils
import "../databasemodule_v2.js" as DB

UbuntuShape
{
    id: articleOneImgA

    property var imageArray: []
    property bool invalid: false
    property var modelItem
    property var rssModel
    property int modelIndex

    width: units.gu(18)
    height: units.gu(33) /*content.height + units.gu(1.5)*/
    radius: "medium"
    color: invalid ? "black" : modelItem.status == "1" ? Qt.rgba(0, 0, 0, 0.7) : Qt.rgba(0, 0, 0, 0.4)
    clip: true

    onModelItemChanged: {
        if (modelItem == null) {
            invalid = true
        }
    }

    Item {
        id: shapeClipper
        width: parent.width
        height: uPic.height > (articleOneImgA.height * 0.75) ? (articleOneImgA.height * 0.75) : uPic.height
        clip: true

        UbuntuShape {
            id: uPic

            width: parent.width
            height: pic.height /*> units.gu(12) ? pic.height : units.gu(12)*/
            radius: "medium"
            opacity: height
            image: Image {
                id: pic
                width: {
                    if (implicitHeight < 50 || implicitWidth < 50) return 0
                    else return uPic.width
                }
                height: width * implicitHeight / implicitWidth
                source: imageArray[0]
                sourceSize.width: uPic.width
            }

            Behavior on height { UbuntuNumberAnimation{} }
            Behavior on opacity { NumberAnimation{} }
        }
    }

    Column {
        anchors {
            left: parent.left; right: parent.right;
            top: shapeClipper.bottom; bottom: parent.bottom
            topMargin: units.gu(2); bottomMargin: units.gu(1.5);
            leftMargin: units.gu(1); rightMargin: units.gu(1.5)
        }
        spacing: units.gu(1)

        Label {
            id: labelTime
            text: { invalid ? "" : DateUtils.formatRelativeTime(i18n, modelItem.pubdate) }
            fontSize: "x-small"
            width: parent.width
            wrapMode: Text.WrapAtWordBoundaryOrAnywhere
            opacity: 0.6
        }

        Label {
            id: labelTitle

            text: invalid ? "" : modelItem.title
            width: parent.width
            height: parent.height - parent.spacing * 2 - labelTime.paintedHeight - labelTeedname.paintedHeight
            wrapMode: Text.WrapAtWordBoundaryOrAnywhere
            fontSize: "small"
            textFormat: Text.PlainText
            font.weight: Font.DemiBold
            elide: Text.ElideRight
            opacity: invalid ? 0.4 : modelItem.status == "1" ? 0.4 : 0.8
        }

        Label {
            id: labelTeedname

            text: invalid ? "" : modelItem.feed_name
            fontSize: "x-small"
            width: parent.width
            wrapMode: Text.WrapAtWordBoundaryOrAnywhere
            opacity: 0.6
        }
    }
    //    }

    MouseArea {
        anchors.fill: parent
        onClicked: {
            mainView.toRssPage(rssModel, modelIndex)
        }
    }
}
