import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1

import "../databasemodule_v2.js" as DB

Page {
    id: editFeedsPageRoot
    objectName: "editfeedspage"
    flickable: null
    title: i18n.tr("Feeds")

    property var currentTask: null

    /* Refresh list with sections.
             */
    function reloadPageContent() {
        topicList.model = null // Sections are not recreated without this line.
        topicListModel.clear()

        /* Append feeds with topics.
                 */
        var allFeeds = DB.loadFeeds()
        var tags = DB.loadTags()

        for (var i = 0; i < tags.rows.length; i++) {
            var curTag = tags.rows.item(i) // TMP
            var feedsByTag = DB.loadFeedsFromTag(curTag.id)

            for (var j = 0; j < feedsByTag.rows.length; j++) {
                var curFeed = feedsByTag.rows.item(j)
                topicListModel.append({"tagName":curTag.name,
                                          "tagId" : curTag.id,
                                          "feedName" : curFeed.title,
                                          "feedId" : curFeed.id,
                                          "status" : curFeed.status,
                                          "feedSource": curFeed.source,
                                          "isSelected" : false })
            }
        }

        /* Append rest of feeds.
                 */
        var feedTags = DB.loadFeedTags()

        for (var j = 0; j < allFeeds.rows.length; j++) {
            var curFeed = allFeeds.rows.item(j)

            var found = false
            for (var i = 0; i < feedTags.rows.length; i++) {
                if (curFeed.id == feedTags.rows.item(i).feed_id) {
                    found = true
                    break
                }
            }

            /* Feed without topic!
                     */
            if (!found) {
                topicListModel.append({"tagName": i18n.tr("No topic"),
                                          "tagId" : 0,
                                          "feedName" : curFeed.title,
                                          "feedId" : curFeed.id,
                                          "status" : curFeed.status,
                                          "feedSource": curFeed.source,
                                          "isSelected" : false })
            }
        }

        topicList.model = topicListModel
    } // reloadList

    /* Operations on empty selection are useless.
             * Helper function for checking this.
             */
    function checkEmpty() {
        var haveSel = false
        for (var i = 0; i < topicListModel.count; i++) {
            if (topicListModel.get(i).isSelected) {
                haveSel = true
                break
            }
        }

        if (!haveSel) {
            console.log("EMPTY SELECTION")
        }

        return !haveSel
    }



    tools: ToolbarItems {
        id: editTopicToolbar

        ToolbarButton {
            id: editTopicsToolButton

            action: Action {
                text:  i18n.tr("Topics")
                iconSource: Qt.resolvedUrl("../avatar.png")
                onTriggered: {
                    manageTopicsPage.reloadPageContent()
                    pageStack.push(manageTopicsPage)
                }
            }
        }

//        ToolbarButton {
//            id: addFeedToolButton

//            action: Action {
//                text:  i18n.tr("Add feed")
//                iconSource: Qt.resolvedUrl("../avatar.png")
//                onTriggered: {
//                    /* When we want to show "Append feed" page from
//                     * any page except of "Choose topic",
//                     * we should clear it (mark as dirty).
//                     */
//                    pageStack.push(appendFeedPage, {"isDirty" : true})
//                }
//            }
//        }

        ToolbarButton {
            id: moveToolButton

            action: Action {
                text:  i18n.tr("Move to...")
                iconSource: Qt.resolvedUrl("../avatar.png")
                onTriggered: {
                    if (editFeedsPageRoot.checkEmpty())
                        return
                    PopupUtils.open(topicSelectActionPopoverComponent, moveToolButton)
                }
            }
        }

        ToolbarButton {
            id: deselectToolButton

            action: Action {
                text:  i18n.tr("Deselect")
                iconSource: Qt.resolvedUrl("../avatar.png")
                onTriggered: {
                    for ( var i = 0; i < topicListModel.count; i++) {
                        topicListModel.setProperty(i, "isSelected", false)
                    }
                }
            }
        }
    } // ToolBarItems

    Component {
        id: topicSelectActionPopoverComponent

        ActionSelectionPopover {
            id: topicSelectActionPopover

            Component.onCompleted: {

                var displayData = []
                var tags = DB.loadTags()
                for(var i = 0; i < tags.rows.length; i++) {
                    var curTag = tags.rows.item(i)
                    displayData.push({"text" : curTag.name, "tagId" : curTag.id})
                }

                displayData.push({"text" : i18n.tr("No topic"), "tagId" : 0})

                var readyActions = []
                for(var i = 0; i < displayData.length; i++) {
                    var obj = myAction.createObject(topicSelectActionPopover)
                    obj.text = displayData[i].text
                    obj.tagId = displayData[i].tagId
                    obj.topicSelected.connect(topicSelectedEvent)
                    readyActions.push(obj)
                }

                topicSelectActionPopover.actions = readyActions
            }

            /* Processing move or copy request.
                     */
            function topicSelectedEvent(tagId) {
                PopupUtils.close(topicSelectActionPopover)

                for (var i = 0; i < topicListModel.count; i++) {
                    var entry = topicListModel.get(i)

                    if (entry.isSelected) {
                        if (topicSelectActionPopover.isMove)
                            DB.deleteFeedTag(entry.feedId, entry.tagId)
                        if (tagId !== 0) {
                            DB.addFeedTag(entry.feedId, tagId)
                        } else console.log("Can't move to NO TOPIC")
                    }
                }

                editFeedsPageRoot.reloadPageContent()
            }
        }
    }

    Component {
        id: myAction

        Action {
            property int tagId: 0
            signal topicSelected(int tagId)

            onTriggered: {
                topicSelected(tagId)
            }
        }
    }

    /* Sectioned list of topics (tags).
             */
    ListView {
        id: topicList

        anchors.fill: parent
        // anchors.topMargin: units.gu(2)
        clip: true
        model: topicListModel

        header: Column {
            width: parent.width
            spacing: units.gu(2)

            Item { // Just for alignment.
                id: dummyItem
                height: btnAddTopic.height + units.gu(2)
                width: parent.width

                Button {
                    id: btnAddTopic
                    text: i18n.tr("Append feed")
                    width: units.gu(20)
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.bottom: parent.bottom

                    onClicked: {
                        pageStack.push(appendFeedPage, {"isDirty" : true})
                    }
                }
            }

            ListItem.ThinDivider { }
        }

        delegate: ListItem.Standard {
            objectName: "FeedListItem"
            text: ((model.feedName == "") ? model.feedSource : model.feedName)
            selected: model.isSelected

            onClicked: {
                topicListModel.get(index).isSelected = !topicListModel.get(index).isSelected
            }

            onPressAndHold: {
                topicList.currentIndex = model.index
                PopupUtils.open(popoverComponent, topicList.currentItem) //topicList)
            }
        }

        section {
            property: "tagId"
            criteria: ViewSection.FullString
            delegate: ListItem.Header {
                // height: units.gu(5)
                Component.onCompleted: {
                    var s = section
                    for (var i = 0; i < topicListModel.count; i++) {
                        if (topicListModel.get(i).tagId == s) {
                            text = topicListModel.get(i).tagName
                            break
                        }
                    }
                }
            }
        }
    } // ListView

    ListModel {
        id: topicListModel
    }

    Label {
        visible: topicListModel.count == 0
        anchors.centerIn: parent
        text: i18n.tr("Use toolbar to append feeds")
        fontSize: "large"
    }

    Component {
        id: popoverSheetComponent

        ComposerSheet {
            id: appendSheet

            property bool isForEdit: false
            property var itemData: null

            Component.onCompleted: {
                if (itemData) {
                    tfFeedUrl.text = itemData.feedName
                }

                tfFeedUrl.forceActiveFocus()
                tfFeedUrl.cursorPosition = tfFeedUrl.text.length
            }

            onConfirmClicked: {
                if (isForEdit) {
                    if (itemData.status == '1') {
                        if (tfFeedUrl.text != "") {
                            itemData.feedName = tfFeedUrl.text
                            DB.updateFeedByUser(itemData.feedId, itemData.feedName, itemData.feedSource)
                        }
                    }
                }
            }

            onCancelClicked: {
                console.log("Canceled")
            }

            contentsWidth: sheetContentColumn.width
            contentsHeight: units.gu(11) // TODO BUG - doesn't works
            title: isForEdit? i18n.tr("Edit feed") : i18n.tr("Add feeds")

            Column {
                id: sheetContentColumn
                spacing: units.gu(2)
                width: units.gu(30)
                anchors.centerIn: parent

                TextField {
                    id: tfFeedUrl
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width
                    placeholderText: isForEdit? i18n.tr("Type a new name here...") : i18n.tr("Type a URL here...")
                }

                Item {
                    width: parent.width
                    height: units.gu(5)

                    Label {
                        anchors {
                            left: parent.left
                            verticalCenter: parent.verticalCenter
                            leftMargin: units.gu(1)
                        }
                        text: i18n.tr("Show in inbox")
                    }

                    Switch {
                        id: swShowOnMainscreen
                        checked: true
                        anchors {
                            right: parent.right
                            verticalCenter: parent.verticalCenter
                            rightMargin: units.gu(1)
                        }
                    }
                }
            } // Column
        } // ComposerSheet
    } // Component popoverSheetComponent

    Component {
        id: popoverComponent

        /* This new popover works like context menu.
                 * Actions can be reused in toolbar.
                 */
        ActionSelectionPopover {
            id: actionSelectPopover
            actions: ActionList {
                id: popoverActionsList

                Action {
                    text: i18n.tr("Edit")
                    onTriggered: {
                        // PopupUtils.close(actionSelectPopover)
                        var data = topicListModel.get(topicList.currentIndex)
                        console.log("START EDIT ", data.source, data.status)
                        if (data.status == '0') {
                            console.log("EDIT ABORT")
                            return
                        }

                        PopupUtils.open(popoverSheetComponent, tools, {"isForEdit":true, "itemData" : data})
                    }
                }

                Action {
                    text: i18n.tr("Remove")
                    onTriggered: {
                        // PopupUtils.close(actionSelectPopover)
                        var id = topicListModel.get(topicList.currentIndex).feedId
                        var dbResult = DB.deleteFeed(id)
                        if (dbResult.rowsAffected > 0) {
                            DB.clearArticles(id)
                            topicListModel.remove(topicList.currentIndex)
                        }
                    }
                }
            } // ActionList
        } // ActionSelectionPopover
    } // Popover component

} // Page
