import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import Ubuntu.Layouts 0.1

import "../databasemodule_v2.js" as DB

Page {
    objectName: "topicmanagement"
    id: topicManagement

    title: i18n.tr("Management")
    flickable: content
    tools: toolbar

    Component.onCompleted:
    {
        reloadTopics ()
    }

    function reloadTopics ()
    {
        topicModel.clear()
        var topics = DB.loadTags()
        for (var i=0; i < topics.rows.length; i++){
            topicModel.append(
                    {
                "id": topics.rows[i].id
                ,"name": topics.rows[i].name
            })
        }
    }

    function removeModelItem (modelIndex){
        topicModel.remove(modelIndex)
    }

    function reloadPageContent() {

    }

    /*
      toolbar for normal state
    */
    ToolbarItems {
        id: toolbar

        ToolbarButton {
            id: actionsButton
            text: i18n.tr("Add Feed")
            iconSource: Qt.resolvedUrl("../icons_tmp/add.svg")
            onTriggered: pageStack.push(appendFeedPage/*, {"isDirty" : true}*/) ;
            visible: true
        }
    }

    /*
      toolbar for editing state
    */
    ToolbarItems {
        id: editConfirm
        opened: true
        locked: true
        visible: false
//        anchors.bottom: content.bottom

        property int editIndex

        back: Button {
            id: cancelButton
            text: i18n.tr("Cancel")
            gradient: UbuntuColors.greyGradient
            anchors.verticalCenter: parent.verticalCenter
            onClicked: {
                topicList.currentItem.cancelEdit()
                topicManagement.state = ""
                editConfirm.visible = false
                topicList.currentIndex = -1
            }
        }

        Button {
            id: confirmButton
            text: i18n.tr("Confirm")
            anchors.verticalCenter: parent.verticalCenter
            onClicked: {
                topicList.currentItem.confirmEdit()
                topicManagement.state = ""
                editConfirm.visible = false
                topicList.currentIndex = -1
            }
        }
    }

    /*
      main content
    */
    Flickable {
        id: content
        anchors.fill: parent
        contentHeight: contentItem.childrenRect.height
        boundsBehavior: (contentHeight > topicManagement.height) ? Flickable.DragAndOvershootBounds : Flickable.StopAtBounds

        Column{
            anchors{ left: parent.left; right: parent.right }

            /*
              add topic button
            */
            Item {
                id: itemAddTopic
                objectName: "addTopic"
                anchors{ left: parent.left; right: parent.right }
                height: units.gu(8)

                UbuntuShape{
                    id: uAddTopic
                    anchors { fill: parent; margins: units.gu(1) }
                    gradientColor: "goldenrod"
                    radius: "medium"

                    Label{
                        text: i18n.tr("Add a Topic")
                        fontSize: "large"
                        anchors.centerIn: parent
                    }

                    MouseArea{
                        id: maAddTopic
                        anchors.fill: parent
                        onClicked: pageStack.push(createTopicPage)
                    }

                    states: [
                        State {
                            name: "pressed"
                            when: maAddTopic.pressed

                            PropertyChanges
                            {
                                target: uAddTopic
                                gradientColor: "grey"
                            }
                        }
                    ]
                }
            }

            /*
              topic listview
            */
            ListView {
                id: topicList
                anchors{ left: parent.left; right: parent.right }
                height: contentItem.childrenRect.height
                interactive: false
                currentIndex: -1
//                clip: true

                signal collapseAllItem()

                function collapseAll(){
                    collapseAllItem()
                }

                displaced: Transition {
                    NumberAnimation { properties: "x,y"; easing.type: Easing.OutQuad }
                }

                model: VisualDataModel {
                    id: visualModel
                    model: ListModel {
                        id: topicModel
                    }
                    delegate: MouseArea {
                        id: delegateRoot

                        property int visualIndex: VisualDataModel.itemsIndex
                        property alias isExpended: topicItem.isExpended

                        anchors{ left: parent.left; right: parent.right }
                        height: topicItem.height

                        function cancelEdit() {
                            topicItem.cancelEdit()
                        }

                        function confirmEdit() {
                            topicItem.confirmEdit()
                        }

                        TopicComponent{
                            id: topicItem
                            topicName: model.name
                            topicId: model.id
                            modelIndex: index
//                            isExpended: delegateRoot.isExpended

                            onEdit: {
                                if (topicList.currentItem)
                                    topicList.currentItem.cancelEdit()
                                editConfirm.editIndex = index
                                topicManagement.state = "editMode"
                                editConfirm.visible = true
                                topicList.currentIndex = index
                            }

                            onEditCanceled: {
                                if (topicList.currentItem)
                                    topicList.currentItem.cancelEdit()
                                topicManagement.state = ""
                                editConfirm.visible = false
                                topicList.currentIndex = -1
                            }
                        }

                        DropArea {
                            anchors { fill: parent; margins: 0 }

                            onEntered:{
//                                console.log("enter", drag.source)
                                topicItem.selected = true
                            }

                            onExited:{
//                                console.log("exit", drag.source)
                                topicItem.selected = false
                            }

                            onDropped:{
//                                console.log("drop", drag.source)
                                topicItem.selected = false
                                drag.source.opacity = 0
                                console.log("dropped item feed id", drag.source.feedId)
                                console.log("dropped item topic id", drag.source.topicId)
                                DB.deleteFeedTag(drag.source.feedId, drag.source.topicId)
                                DB.addFeedTag(drag.source.feedId, model.id)
                                delegateRoot.isExpended = true
                            }
                        }

                        Connections{
                            id: connTopicList
                            target: topicList

                            onCollapseAllItem:
                            {
                                delegateRoot.isExpended = false
                            }
                        }

                        Connections{
                            id: connEditFeed0
                            target: editFeed

                            onApply:{
                                if (model.id == newTopicId || model.id == previousTopicId){
                                    topicItem.reloadFeed()
                                    topicItem.isExpended = true
                                }
                            }

                            onDeleteFeed:{
                                if (model.id == topicId){
                                    topicItem.reloadFeed()
                                    topicItem.isExpended = true
                                }
                            }
                        }
                    }
                }
            } // listview

        } // column
    } // flickable

    Connections{
        id: connAddTopic
        target: createTopicPage

        onTopicAdded:{
            reloadTopics()
        }
    }

    Connections {
        id: connAddFeed
        target: chooseTopicPage

        onTopicChoosen: {
            reloadTopics()
            for (var i=0; i<topicModel.count; i++){
                if (topicModel.get(i).id == topicId){
                    topicList.currentIndex = i
                    topicList.currentItem.isExpended = true
                }
            }
        }
    }

    states: [
        State {
            name: ""
            PropertyChanges {
                target: topicManagement
                tools: toolbar
            }
        },
        State {
            name: "editMode"
            PropertyChanges {
                target: topicManagement
                tools: editConfirm
            }
        }
    ]

}
