import QtQuick 2.0
import QtQuick.XmlListModel 2.0

import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItems
import Ubuntu.Components.Popups 0.1

import "../databasemodule_v2.js" as DB
import "../dateutils.js" as DateUtils
import "../."

Page {
    id: pageItself

    property bool isDirty: false // Need to clear all page content.

    property int topicId: 0
    property bool isShorts: topicId == 0

    flickable: isShorts ? articleList : articleListByTag

    function reload() {
        isDirty = true
        reloadPageContent()
    }

    function reloadPageContent() {
        listViewModel.clear()

        var tagArray = loadTags()

        if (isShorts) {
            for (var i = 0; i < tagArray.length; i++) {
                var curTag = tagArray[i]

                var tagArticles = DB.loadArticles({"tagId" : curTag.id})
                // console.log("THERE ARE", tagArticles.rows.length, "ARTICLES IN TAG", curTag.id, curTag.name)

                var articlesByTag = []
                for (var j = 0; j < tagArticles.rows.length; j++) {
                    var ca = tagArticles.rows.item(j)

                    var articleObj = {"tagName" : curTag.name,
                        "tagId" : curTag.id,
                        "title" : ca.title,
                        "content" : ca.content,
                        "link" : ca.link,
                        "description" : ca.description,
                        "pubdate" : ca.pubdate,
                        "status" : ca.status, // ?
                        "favourite" : ca.favourite,
                        "feedId" : ca.feed_id,
                        "image" : ca.image,
                        "feed_name" : ca.feed_name,
                        "id" : ca.id }
                        // "guid" : ca.guid

                    articlesByTag.push(articleObj)
                }

                var k = 0
                var limit = Math.min(articlesByTag.length, 8)
                while (k < limit) {
                    listViewModel.append(articlesByTag[k])
                    k++
                }
            }
        } else { // Topic view
            var aTag = tagArray[0]

            // aTag.id == -2 means that current tab is "Saved" tab.
            var topicArticles = ( aTag.id == -2 ) ? DB.loadFavouriteArticles() :
                                                    DB.loadArticles({"tagId" : aTag.id})

            var articlesByTopic = []
            for (j = 0; j < topicArticles.rows.length; j++) {
                ca = topicArticles.rows.item(j)

                var anObj = {"tagName" : aTag.name,
                    "tagId" : aTag.id,
                    "title" : ca.title,
                    "content" : ca.content,
                    "link" : ca.link,
                    "description" : ca.description,
                    "pubdate" : ca.pubdate,
                    "status" : ca.status, // ?
                    "favourite" : ca.favourite,
                    "feedId" : ca.feed_id,
                    "image" : ca.image,
                    "feed_name" : ca.feed_name,
                    "id" : ca.id }
                    // "guid" : ca.guid

                articlesByTopic.push(anObj)
            }

            articlesByTopic.sort(function(a,b) {
                // TODO Mb replace by feed_name comparsion as strings.
                if (a.feedId === b.feedId)
                    return b.pubdate - a.pubdate
                else return b.feedId - a.feedId
            })

            k = 0
            limit = articlesByTopic.length
            while (k < limit) {
                listViewModel.append(articlesByTopic[k])
                k++
            }
        }
    } // reloadPageContent

    function loadTags() {
        if (isShorts) {
            var tags = DB.loadTags()
            var tagArray = []
            for (var j = 0; j < tags.rows.length; j++) {
                tagArray.push(tags.rows.item(j))
            }
            return tagArray
        } else {
            if (topicId == -2) // Need to load saved articles.
                return [{"id" : -2, "name" : ""}]
            else return [{"id" : topicId, "name" : ""}]
        }
    }

    ListModel {
        id: listViewModel
    }

    Component {
        id: listModeDelegate

        ListItems.Subtitled {
            text: model.title
            subText: DateUtils.formatRelativeTime(i18n, model.pubdate) + ", " + model.feed_name
            progression: false
            icon: selectIcon()
            iconFrame: true

            function selectIcon() {
                var img = model.image
                return (model.image == null || model.image == "" ) ? Qt.resolvedUrl("../rssreader64.png"): model.image
            }

            onClicked: {
                mainView.toRssPage(listViewModel, model.index)
            }
        } // delegate
    } // Component

    ListView {
        id: articleList

        clip: true
        anchors.fill: parent
        visible: isShorts
        model: isShorts ? listViewModel : null

        delegate: listModeDelegate

        section {
            criteria: ViewSection.FullString
            property: "tagId"

            delegate: ListItems.Header {
                height: units.gu(5)
                text: textBySection()

                function textBySection() {
                    var s = section
                    for (var i = 0; i < listViewModel.count; i++) {
                        if (listViewModel.get(i).tagId == s) {
                            return listViewModel.get(i).tagName
                        }
                    }
                    // console.log("TEXT DON'T FOUND FOR SECTION ", s)
                    return ""
                }
            }
        } // section
    } // ListView

    ListView {
        id: articleListByTag

        clip: true
        anchors.fill: parent
        visible: !isShorts
        model: isShorts ? null : listViewModel

        delegate: listModeDelegate

        section {
            criteria: ViewSection.FullString
            property: "feedId"

            delegate: ListItems.Header {
                height: units.gu(5)
                text: textBySection()

                function textBySection() {
                    var s = section
                    for (var i = 0; i < listViewModel.count; i++) {
                        if (listViewModel.get(i).feedId == s) {
                            return listViewModel.get(i).feed_name
                        }
                    }
                    // console.log("TEXT DON'T FOUND FOR SECTION ", s)
                    return ""
                }
            }
        } // section
    } // ListView

    Label {
        id: lblEmptyBase

        text: i18n.tr("There are no articles to show")
        anchors.centerIn: parent
        visible: listViewModel.count == 0
        fontSize: "large"
    }
} // Page
