import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import "./feeds"
import "./listview"

import "./databasemodule_v2.js" as DB
import "./imgSeparator.js" as ImageUtils

MainView {
    // objectName for functional testing purposes (autopilot-qt5)
    id: mainView
    objectName: "mainView"
    applicationName: "rssreader-app"

    width: units.gu(50)
    height: units.gu(75)

    headerColor: "#3a2c32"
    backgroundColor: "#875864"
    footerColor: "#9b616c"

    Component.onCompleted: {
        //////////////////////////////////////////////////////////////  below DB operation is for avoid empty db
        // DB.dropTable()
        var result = DB.feedsCount()

        if (result.count < 1) {
            DB.addFeed("" , "http://news.yahoo.com/rss/topstories")
            DB.addFeed("" , "http://news.yahoo.com/rss/tech")
            DB.addFeed("" , "http://www.engadget.com/rss.xml")
            DB.addTag("news");
            DB.addTag("tech");
            DB.addTag("engadget")
            DB.addFeedTag(1, 1)
            DB.addFeedTag(2, 2)
            DB.addFeedTag(3, 3)
            refresh()
        } else repeater.reloadTabs()     // load tabs from db
    }

    /* Refresh current topic or all feeds if we are in the Shorts.
     */
    function refresh() {
        var topicId = 0
        if (pageStack.currentPage == tabstabs) {
            var curTab = tabstabs.selectedTab
            topicId = curTab.topicId
        }

        console.log("REFRESH TOPIC", topicId)

        var feeds
        if (topicId !== 0)
            feeds = DB.loadFeedsFromTag(topicId)
        else feeds = DB.loadFeeds()

        var feedarray = []
        for (var i=0; i< feeds.rows.length; i++)
        {
            feedarray.push(feeds.rows.item(i))
        }
        xmlnetwork.updateFeeds(feedarray, topicId)
    }

                                                   // refresh "Saved" Tab
    function refreshSavedTab() {
        repeater.reloadSaved()
    }

    function toRssPage(model, index)     //   go to single article page
    {
        feed_page.setFeed(model, index)
        pageStack.push(feed_page)
    }

    function toEditFeed(feedid, title, url, pTopicId)
    {
        editFeed.setValues(feedid, title, url, pTopicId) ;
        pageStack.push(editFeed) ;
    }

                                                    //  change the topic name if the name changed by management
    function changeTopicName(topicId, newName) {
        repeater.changeTopicName(topicId, newName)
    }

    PageStack {
        id: pageStack

        property bool isListView: optionsKeeper.useListMode()

        anchors.fill: parent

        Component.onCompleted: {
            push(tabstabs)
            //push(feedListPage)
        }

        onCurrentPageChanged: {
            if (currentPage != null) {
                console.log("PageStack onCurrentPageChanged", currentPage.title)
                if (currentPage.reloadPageContent)
                    currentPage.reloadPageContent()
            }
        }

        ToolbarItems {
            id: mainCommonToolbar

            back: ToolbarButton {
                action: Action {
                    text:  i18n.tr("Refresh")
                    iconSource: Qt.resolvedUrl("./icons_tmp/reload.svg")
                    onTriggered:
                    {
                        refresh();
                        mainCommonToolbar.opened = false;
                    }
                }
            }

            ToolbarButton {
                action: Action {
                    id: changeModeAction

                    text:  pageStack.isListView ? i18n.tr("Grid View") : i18n.tr("List view")
                    iconSource: pageStack.isListView ? Qt.resolvedUrl("./icons_tmp/view-fullscreen.svg") : Qt.resolvedUrl("./icons_tmp/view-fullscreen.svg")
                    onTriggered: {
                        pageStack.isListView = !pageStack.isListView
                        optionsKeeper.setUseListMode(pageStack.isListView)
                        mainCommonToolbar.opened = false
                    }
                }
            }

            ToolbarButton {
                id: addReadsBtn
                objectName: "addreadsbutton"
                action: Action {
                    text:  i18n.tr("Add reads")
                    iconSource: Qt.resolvedUrl("./icons_tmp/add.svg")
                    onTriggered: {
                        PopupUtils.open(addReadsPopoverComp, addReadsBtn)
                    }
                }
            }

            ToolbarButton {
                objectName: "edittopicsbutton"
                action: Action {
                    text:  i18n.tr("Edit topics")
                    iconSource: Qt.resolvedUrl("./icons_tmp/edit.svg")
                    onTriggered: {
                        pageStack.push(topicManagement) ;
                    }
                }
            }
        }

        // ************************************************************* ////////////////////////////////////

        Tabs {
            id: tabstabs
            visible: false

            function selectTopic(topicId) {
                if (topicId == 0) {
                    tabstabs.selectedTabIndex = 0
                } else {
                    for(var i = 0; i < repeater.count; i++) {
                        if (repeater.itemAt(i).topicId == topicId) {
                            tabstabs.selectedTabIndex = i + 1
                            return tabstabs.selectedTab
                        }
                    }
                }
            }

            TopicTab {
                id: shortsTab

                objectName: "Tab0"
                title: i18n.tr("Shorts")
                topicId: 0
                isAll: true

                Component.onCompleted: reload()
            }

            Repeater {
                id: repeater

                model: 0
                TopicTab { }

                function reloadTabs() {
                    repeater.model = 0
                    var tags = DB.loadTags()
                    repeater.model = tags.rows.length + 1  // plus 1 for "Saved" tab

                    for (var i = 0; i< tags.rows.length; i++) {
                        repeater.itemAt(i).title = tags.rows.item(i).name
                        repeater.itemAt(i).topicId = tags.rows.item(i).id
                        repeater.itemAt(i).isAll = false
                        repeater.itemAt(i).reload()
                    }
                    // for "Saved" tab (means "favourite" in db)
                    repeater.itemAt(tags.rows.length).title = i18n.tr("Saved")
                    repeater.itemAt(tags.rows.length).topicId = -2
                    repeater.itemAt(tags.rows.length).isAll = true
                    repeater.itemAt(tags.rows.length).reload()
                }

                function reloadSaved() {
                    repeater.itemAt(repeater.model - 1).reload()
                }

                function changeTopicName(topicId, newName) {
                    for(var i = 0; i < repeater.count; i++) {
                        if (repeater.itemAt(i).topicId == topicId) {
                            repeater.itemAt(i).title = newName
                            break
                        }
                    }
                }
            } // Repeater
        }

        // ******************************** Create Topic **********************///////////////

        CreateTopicPage {
            id: createTopicPage
            visible: false
            flickable: null
        } // Page

        // ******************************** APPEND FEED ***********************///////////////

        AppendFeedPage {
            id: appendFeedPage

            title: i18n.tr("Add feeds")
            flickable: null
            visible: false
        }

        // ******************************** Choose Topic Page ***********************///////////////

        ChooseTopicPage {
            id: chooseTopicPage
            visible: false
        }

        // ******************************** Rss Feed Page ***********************///////////////

        RssFeedPage {
            id: feed_page
            visible: false
        }

        TopicManagement {  // by Joey
            id: topicManagement
            visible: false
        }

        EditFeed {   // by Joey
            id: editFeed
            visible: false
        }

        // ************************************************************* ////////////////////////////////////

        // Preserve it temporary please, I frequently use it for testing.
//        FeedListPage {
//            id: feedListPage

//            title: i18n.tr(" ")
//            flickable: null
//            visible: false
//        }

//        ArticleListPage {
//            id: articleListPage

//            title: i18n.tr("Articles")
//            flickable: null
//            visible: false
//        }

    } // PageStack

    Component {
        id: refreshWaitDialogComponent

        Dialog {
            id: refreshWaitDialog
            objectName: "refreshWaitDialog"

            title: i18n.tr("Checking for new articles")

            ActivityIndicator {
                objectName: "activityindicator"
                running: true
            }

            Button {
                text: i18n.tr("Cancel")
                objectName: "refreshCancel"
                color: UbuntuColors.orange
                onClicked: {
                    xmlnetwork.cancelDownload()
                    PopupUtils.close(refreshWaitDialog)
                }
            }

            Connections {
                target: xmlnetwork

                onDownloadFinished: {
                    PopupUtils.close(refreshWaitDialog)
                }
            }
        } // Dialog
    } // Component

    Component {
        id: addReadsPopoverComp

        ActionSelectionPopover {
            id: addReadsPopover
            objectName: "addreadspopover"

            actions: ActionList {
                id: popoverActionsList

                Action {
                    text: i18n.tr("+ Add feeds")
                    onTriggered: {
                        /* When we want to show "Append feed" page from
                         * any page except of "Choose topic",
                         * we should clear it (mark as dirty).
                         */
                        pageStack.push(appendFeedPage, {"isDirty" : true})
                    }
                }

                // Currently redundant. We will use it later.
//                Action {
//                    text: i18n.tr("+ New topic")
//                    onTriggered: {
//                        pageStack.push(createTopicPage)
//                    }
//                }

                Action {
                    text: i18n.tr("Add online accounts")
                    onTriggered: {
                        console.log("NOT IMPLEMENTED")
                    }
                }

                Action {
                    text: i18n.tr("Import subscriptions")
                    onTriggered: {
                        console.log("NOT IMPLEMENTED")
                    }
                }
            } // ActionList
        } // ActionSelectionPopover
    } // Component

    Component {
        id: errorDialogComponent

        Dialog {
            id: errorDialog

            Button {
                text: i18n.tr("Ok")
                objectName: "errordialogbutton"
                onClicked: PopupUtils.close(errorDialog)
            }
        }
    } // Component

    function reloadViews(tagId) {
        shortsTab.reload()
        repeater.reloadTabs()
        tabstabs.selectTopic(tagId)
        // TODO use later.
        // Weak and slow place, we need dynamic tabs management to speed up.
        //        if (tagId == 0) {
        //            shortsTab.reload()
        //            repeater.reloadTabs()
        //        } else {
        //            var tab = tabstabs.selectTopic(tagId)
        //            tab.reload()
        //        }
    }

    XmlNetwork {
        id: xmlnetwork

        onDownloadFinished: {
            reloadViews(tagId)
        }

        onDownloadStarted: {
            PopupUtils.open(refreshWaitDialogComponent, null)
        }
    }

    OptionsKeeper {
        id: optionsKeeper
    }

    ////////////////////////////////////////////////////////////////////////
    //////////// Connections for handling the add topic and add feed event
    Connections
    {
        id: connAddTopic
        target: createTopicPage

        onTopicAdded: {
            reloadViews()
        }
    }

    Connections
    {
        id: connAddFeed
        target: chooseTopicPage

        onTopicChoosen: {
            xmlnetwork.updateFeeds(addedFeeds, topicId)
        }
    }

    // Workaround for https://bugs.launchpad.net/ubuntu-ui-toolkit/+bug/1201400.
    // Inspired by the KeyboardRectangle component, originally written for the
    // telephony application.
    Connections {
        target: Qt.inputMethod

        onVisibleChanged: {
            console.log("keyboard visible: ", Qt.inputMethod.visible)
            if (!Qt.inputMethod.visible) {
                var focusedItem = recursiveFindFocusedItem(window);
                if (focusedItem != null) {
                    focusedItem.focus = false;
                }
            }
        }

        function recursiveFindFocusedItem(parent) {
            if (parent.activeFocus) {
                return parent;
            }

            for (var i in parent.children) {
                var child = parent.children[i];
                if (child.activeFocus) {
                    return child;
                }

                var item = recursiveFindFocusedItem(child);

                if (item != null) {
                    return item;
                }
            }

            return null;
        }
    }
}
