# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""RSS Reader app autopilot tests."""

from __future__ import absolute_import


import unittest

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals, Is, Not

from autopilot.introspection.dbus import StateNotFoundError
from ubuntu_rssreader_app.tests import RssReaderAppTestCase
import logging

from time import sleep

class TestMainWindow(RssReaderAppTestCase):

    global logger
    logger = logging.getLogger(__name__)

    def setUp(self):
        super(TestMainWindow, self).setUp()

        self.assertThat(self.main_view.visible, Eventually(Equals(True)))

        #wait for any updates to finish before beginning tests
        try:
            self.main_view.get_activityIndicator().running.wait_for(False)
        except:
            self.assertThat(
                self.main_view.get_topics_page, Eventually(NotEquals(None)))

    def _add_feed_and_topic(self, feed, topicName):
        """test add feed and topic"""
        logger.debug("starting test add feed and topic")

        self.assertThat(self.main_view.get_topics_page, Eventually(NotEquals(None)))

        #open toolbar
        logger.debug("opening toolbar")
        toolbar = self.main_view.open_toolbar()

        #click on add reads toolbar button
        logger.debug("clicking addreadsbutton")
        toolbar.click_button("addreadsbutton")

        #click on add feeds popover action
        logger.debug("clicking + Add feeds")
        addReadsPopover = self.main_view.get_action_popover()
        self.main_view.run_action(addReadsPopover, "+ Add feeds")

        #verify Append Feed  page is open
        logger.debug("asserting append page")
        appendpage = lambda: self.main_view.get_append_page().visible
        self.assertThat(appendpage, Eventually(Equals(True)))

        #type feed in input field
        logger.debug("inputting feed")
        self._input_new_feed(feed)

        #click on checkbox to confirm feed
        logger.debug("asserting checkbox")
        self.assertThat(self.main_view.get_append_page_feed, Eventually(Not(Is(None))))
        logger.debug("grabbing checkbox")
        checkboxBtn = self.main_view.get_append_page_feed()
        logger.debug("clicking checkbox")
        self.pointing_device.click_object(checkboxBtn)
        logger.debug("asserting checkbox checked")
        self.assertThat(checkboxBtn.checked, Eventually(Equals(True)))

        #click on Next button
        logger.debug("asserting next")
        self.assertThat(self.main_view.get_next_button, Eventually(Not(Is(None))))
        logger.debug("grabbing next")
        nextButton = self.main_view.get_next_button()
        logger.debug("clicking next")
        self.pointing_device.click_object(nextButton)

        #verify choose topic page has opened
        logger.debug("asserting choose topic page")
        chossetopicpage = lambda: self.main_view.get_topics_page().visible
        self.assertThat(chossetopicpage, Eventually(Equals(True)))

        #get current topics
        logger.debug("grabbing topics")
        topics = self.main_view.get_choose_topics_items()
        numTopics = len(topics)
        logger.debug("found " + str(numTopics) + " topics")
        for topic in topics:
            logger.debug("Topic: " + str(topic.text))

        #add a new topic
        logger.debug("input topic")
        self._input_new_topic(topicName)

        #A new topic is now automatically selected as the topic
        #select new topic
        #logger.debug("assert new topic")
        #newNumTopics = len(self.main_view.get_choose_topics_items)
        #self.assertThat(newNumTopics, Eventually(Equals(numTopics + 1)))
        #logger.debug("grab new topic")
        #topics = self.main_view.get_choose_topics_items()
        #numTopics = len(topics)
        #logger.debug("found " + str(numTopics) + " topics")
        #for topic in topics:
        #    logger.debug("Topic: ") + str(topic.text)
        #newTopic = self.main_view.get_choose_topics_items()[-1]
        #newTopic = topics[-1]
        #logger.debug("click new topic, ") + str(newTopic.text)
        #self.pointing_device.click_object(newTopic)
        #logger.debug("topic clicked")

        #wait for any updates to finish before beginning tests
        #this transition currently locks the device https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1221893
        #logger.debug("check for network updates")
        #self.assertThat(self.main_view.get_network_activity, Eventually(Not(Is(None))))
        #logger.debug("assert no network updates")
        #self.assertThat(self.main_view.get_network_activity, Eventually(Is(None)))
        #logger.debug("assert we're back on main view")
        #self.assertThat(self.main_view.visible, Eventually(Equals(True)))
        #logger.debug("finished with add_feed_topic")

    def _remove_feed(self, feedName, topicName):
        logger.debug("removing feed")
        #open toolbar
        logger.debug("opening toolbar")
        toolbar = self.main_view.open_toolbar()

        #click on back toolbar Button
        logger.debug("clicking edit topics")
        toolbar.click_button("edittopicsbutton")

        #verify we are on the Management page
        logger.debug("assert feedlist page")
        feedlistpage = lambda: self.main_view.get_feed_management_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #select topic
        logger.debug("grabbing topic")
        self._get_topic(topicName)

        #select feed
        logger.debug("assert feed")
        self.assertThat(lambda: self.main_view.get_feedlist_feed(feedName), Eventually(Not(Is(None))))
        logger.debug("grabbing feed")
        feed = self.main_view.get_feedlist_feed(feedName)
        logger.debug("clicking feed")
        self.pointing_device.click_object(feed)

        #verify we are on the Edit Feed page
        logger.debug("assert editfeedpage")
        editFeedPage = self.main_view.get_edit_feed_page()
        self.assertThat(editFeedPage.visible, Eventually(Equals(True)))

        #press delete button
        logger.debug("assert delete button")
        self.assertThat(self.main_view.get_delete_button, Eventually(Not(Is(None))))
        logger.debug("grab delete button")
        deleteButton = self.main_view.get_delete_button()
        logger.debug("click delete button")
        self.pointing_device.click_object(deleteButton)

        #verify we are on the Management page
        logger.debug("assert feedlistpage")
        feedlistpage = lambda: self.main_view.get_feed_management_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #check topic
        logger.debug("assert topic remains")
        self.assertThat(lambda: self.main_view.get_feedlist_topic(topicName), Eventually(Not(Is(None))))

        #check feed has been removed
        logger.debug("assert removed feed")
        self.assertThat(lambda: self.main_view.get_feedlist_feed(feedName), Eventually(Equals(None)))

    def _remove_topic(self, topicName):
        #this been quasi converted to the sdk, but does not work as-is
        #see https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1211631

        self.assertThat(self.main_view.get_topics_page, Eventually(NotEquals(None)))

        #open toolbar
        toolbar = self.main_view.open_toolbar()
        self.assertThat(self.main_view.open_toolbar, Eventually(NotEquals(None)))

        #click on back toolbar Button
        backButton = self.main_view.get_toolbar_back_button()
        self.pointing_device.click_object(backButton)
        self.assertThat(self.main_view.get_topics_page, Eventually(NotEquals(None)))

        #open toolbar
        toolbar = self.main_view.open_toolbar()
        self.assertThat(self.main_view.open_toolbar, Eventually(NotEquals(None)))

        #click on edit topics button
        toolbar.click_button("edittopicsbutton")

        #verify we are on the Management page
        feedlistpage = self.main_view.get_feed_management_page()
        self.assertThat(feedlistpage.visible, Eventually(Equals(True)))

        #select topic
        topic = self._get_topic(topicName)

        #topicsPage = self.main_view.get_manage_topics_page()
        #self.assertThat(self.main_view.get_manage_topics_page, Eventually(NotEquals(None)))
        topicmanagementpage = self.main_view.get_feed_management_page()
        self.assertThat(self.main_view.get_feed_management_page, Eventually(NotEquals(None)))

        #swipe to delete canonical topic
        startX = int(topicmanagementpage.x + topicmanagementpage.width * 0.30)
        stopX = int(topicmanagementpage.x + topicmanagementpage.width)
        lineY = topic.globalRect[1]

        #swipe to remove the topic
        self.pointing_device.move(startX, lineY)
        self.pointing_device.click()
        self.pointing_device.drag(startX, lineY, stopX, lineY)

        #verify we are on the Management page  *** for timing issues ***
        feedmanagementpage = self.main_view.get_feed_management_page()
        self.assertThat(feedmanagementpage, NotEquals(None))

        #check that it is gone
        topic = self.main_view.get_feedlist_topic(topicName)
        self.assertThat(topic, NotEquals(topicName))

    def _input_new_feed(self, rssFeed):
        inputField = self.main_view.get_append_feed_input_box()
        return self._safe_enter_text(inputField, rssFeed)

    def _input_new_topic(self, topic):
        inputField = self.main_view.get_new_topic_input_box()
        return self._safe_enter_text(inputField, topic)

    def _safe_enter_text(self, inputField, text, timeout = 10):
        #poll clicking for focus for lack of better method, due to screen switching animation
        #is there a property we can use instead?
        poll = 0
        while inputField.focus != True and poll < timeout:
            self.pointing_device.click_object(inputField)
            sleep(1)
            poll += 1
        self.keyboard.type(text)
        self.assertThat(inputField.text, Eventually(Equals(text)))
        self.keyboard.press_and_release("Enter")
        return inputField

    def _get_topic(self, topicName, timeout = 10):
        logger.debug("_get_topic: assert topics")
        self.assertThat(lambda: self.main_view.get_feedlist_topic(topicName), Eventually(Not(Is(None))))
        logger.debug("_get_topic: grab topic")
        topic = self.main_view.get_feedlist_topic(topicName)
        logger.debug("_get_topic: click topic")

        poll = 0
        while topic.isExpended == False and poll < timeout:
            logger.debug("_get_topic: loop click topic")
            self.pointing_device.click_object(topic)
            sleep(1)
            poll += 1
        return topic

    def test_add_remove_feed_and_topic(self):
        """ test add and remove feed and topic"""

        #add topic and feed
        #TODO, try several different feeds
        feed = "http://www.canonical.com/rss.xml"
        feedName = "Canonical"
        topicName = "CanonicalTopic"

        self._add_feed_and_topic(feed, topicName)

        #remove added feed
        logger.debug("starting remove feed")

        self._remove_feed(feedName, topicName)

        #remove added topic
        self._remove_topic(topicName)


    @unittest.skip("Topic page broken, see https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1206248")
    def test_view_feeds(self):
        """test view feeds"""

        self.assertThat(self.main_view.get_topics_page(), NotEquals(None))

        #-----> view all news:
        #open toolbar
        toolbar = self.main_view.open_toolbar()

        #click on add reads toolbar button
        toolbar.click_button("List view")

        #verify we are on the Feeds page
        feedlistpage = self.main_view.get_feedlist_page()
        self.assertThat(feedlistpage.visible, Eventually(Equals(True)))

        #select All articles
        allFeeds = self.main_view.get_all_feeds()
        self.assertThat(allFeeds, NotEquals("None"))
        self.pointing_device.click_object(allFeeds )

        #verify we are on the All articles page
        articlelistpage = self.main_view.get_articlelist_page()
        self.assertThat(articlelistpage.visible, Eventually(Equals(True)))

        toolbar = self.main_view.open_toolbar()

        #click on back toolbar button
        toolbar.click_button("Back")

        #verify we are on the Feeds page
        feedlistpage = lambda: self.main_view.get_feedlist_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #-----> view individual feed:
        #add topic and feed
        self._add_feed_and_topic()

        #select Canonical feed
        canonicalFeed = lambda: self.main_view.get_canonical_feed_labelvisual()
        self.assertThat(canonicalFeed, Eventually(Not(Is(None))))
        self.pointing_device.click_object(canonicalFeed)

        #verify we are on the Canonical page
        header = lambda: self.main_view.get_header()[0].title
        self.assertThat(header, Eventually(Equals("Canonical")))

        #toggle toolbar
        toolbar = self.main_view.open_toolbar()

        #click on back toolbar button
        toolbar.click_button("Back")

        #verify we are on the Feeds page
        feedlistpage = lambda: self.main_view.get_feedlist_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #toggle toolbar
        toolbar = self.main_view.open_toolbar()

        #click on shorts toolbar button
        toolbar.click_button("shorts")

        #verify we are on the Shorts page
        shortsPage = lambda: self.main_view.switch_to_tab_by_index(0).visible
        self.assertThat(shortsPage, Eventually(Equals(True)))

        #remove added Canonical topic
        self._remove_topic()

        #remove added Canonical feed
        self._remove_feed()
